/*
  lpc8/gpio.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */
#ifndef lpc8__gpio_h
#define lpc8__gpio_h

#include <integers.h>
#include <unionized.h>

union PACKED4 Gpio {
	/** Allows access to single port pins PIO0_0 to PIO0_17 with one byte per pin.
	 * When writing, the port pin p is set if b[p] != 0 and is cleared if b[p]==0.
	 * When reading, the result is either 0xFF (pin is 1) of 0x00 (pin is 0).
	 */
	//.0x0000
	unionized struct PACKED { char _0[0x0000]; // padding
	volatile Uint8		b[18];
	unionized };

	/** Allows access to single port pins PIO0_0 to PIO0_17 with one 32-bit word per pin.
	 * When writing, the port pin p is set if b[p] != 0 and is cleared if b[p]==0.
	 * When reading, the result is either 0xFFFFFFFF (pin is 1) of 0x00000000 (pin is 0).
	 */
	//.0x1000
	unionized struct PACKED { char _1[0x1000]; // padding
	volatile Uint32		w[18];
	unionized };

	/** Pin direction register. If bit p is set, then PIO0_p is an output, otherwise an input.
	 */
	//.0x2000
	unionized struct PACKED { char _2[0x2000]; // padding
	volatile Uint32		dir0;
	unionized };

	/** Masks bits when writing to mPin0. If bit p is set to 1, then PIO0_p is unaffected by a write to mPin0,
	 * otherwise the write appears on PIO0_p.
	 */
	//.0x2080
	unionized struct PACKED { char _3[0x2080]; // padding
	volatile Uint32		mask0;
	unionized };

	/** Value of the pins. When writing, the values appear on the outputs. When reading, the current level of the
	 * pin is returned. This is independent of the mask0 register contents.
	 */
	//.0x2100
	unionized struct PACKED { char _4[0x2100]; // padding
	volatile Uint32		pin0;
	unionized };

	/** Value of the pins, filtered by mask0. When writing, the values appear on the outputs provides the
	 * corresponding mask0 bit is NOT set.
	 * When reading the current level of the pin is returned unless the corresponding bit is mask0 is set. If the
	 * mask0 bit is set, then 0 is returned.
	 */
	//.0x2180
	unionized struct PACKED { char _5[0x2180]; // padding
	volatile Uint32		mPin0;
	unionized };

	/** Sets output pins regardless of the mask0 contents. 1 = set output pin, 0 = do nothing.
	 */
	//.0x2200
	unionized struct PACKED { char _6[0x2200]; // padding
	volatile Uint32		set0;
	unionized };

	/** Clears output pins regardless of the mask0 contents. 1 = clear output pin, 0 = do nothing.
	 */
	//.0x2280
	unionized struct PACKED { char _7[0x2280]; // padding
	volatile Uint32		clr0;
	unionized };

	/** Toggles output pins regardless of the mask0 contents. 1 = toggle pin, 0 = do nothing.
	 */
	//.0x2300
	unionized struct PACKED { char _8[0x2300]; // padding
	volatile Uint32		not0;
	unionized };
};

extern union Gpio GPIO;

#endif

