/*
  lpc8/pim.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

#ifndef lpc8__pim_h
#define lpc8__pim_h

/** @file
 * @brief Pin interrupt and pattern matching engine. 
 */
#include <integers.h>

struct PACKED4 Pim {
	volatile Uint32		isel;		///< interrupt mode, bits 0..7 for pins 0..7
	volatile Uint32		ienr;		///< pin interrupt enable, rising edge, bits 0..7
	volatile Uint32		setIenr;	///< sets bits in ienr
	volatile Uint32		clrIenr;	///< clears bits in ienr

	volatile Uint32		ienf;		///< pin interrupt enable, falling edge, bits 0..7
	volatile Uint32		setIenf;	///< sets bits in ienf
	volatile Uint32		clrIenf;	///< clears bits in ienf
	volatile Uint32		rise;		///< detected rising edges; clear by writing ones.
	
	volatile Uint32		fall;		///< detected falling edges; clear by writing ones.
	volatile Uint32		stat;		///< edge/level status; clear: see manual.
	volatile Uint32		pmCtrl;		///< pin interrupt or pattern match, RXEV on/off
	volatile Uint32		pmSrc;

	volatile Uint32		pmCfg;
};

enum PimIsel {
	PIM_ISEL_EDGE_		=0,
	PIM_ISEL_LEVEL_		=1,
};

/** Used for both rising and falling configuration registers ienr/ienf.
 */
enum PimIen {
	PIM_IEN_DISABLE_	=0,
	PIM_IEN_ENABLE_		=1,
};

enum PimPmCtrl {
	_PIM_PMCTRL_SEL_PMATCH	=0,	///< 1 bit, 0=pin interrupt, 1=pattern match
		PIM_PMCTRL_SEL_PMATCH_INT_	=0,
		PIM_PMCTRL_SEL_PMATCH_PM_	=1,
	_PIM_PMCTRL_ENA_RXEV	=1,	///< 1 bit: enable (1) RXEV output to ARM core/GPIO
		PIM_PMCTRL_ENA_RXEV_DISABLE_	=0,
		PIM_PMCTRL_ENA_RXEV_ENABLE_	=1,
	_PIM_PMCTRL_PMAT	=24,	///< 8 bits current matching product terms output
};

enum PimPmSrc {
	_PIM_PMSRC_0		=8,	///< 3 bits, input source (0..7) for bit slice 0
	_PIM_PMSRC_1		=11,	///< 3 bits, input source for bit slice 1
	_PIM_PMSRC_2		=14,	///< 3 bits, input source for bit slice 2
	_PIM_PMSRC_3		=17,	///< 3 bits, input source for bit slice 3
	_PIM_PMSRC_4		=20,	///< 3 bits, input source for bit slice 4
	_PIM_PMSRC_5		=23,	///< 3 bits, input source for bit slice 5
	_PIM_PMSRC_6		=26,	///< 3 bits, input source for bit slice 6
	_PIM_PMSRC_7		=29,	///< 3 bits, input source for bit slice 7
};

enum PimPmCfg {
	_PIM_PMCFG_ENDP0	=0,	///< 1 bit, 1=bit slice 0 is endpoint of product (minterm), 0=isn't.	
	_PIM_PMCFG_ENDP1	=1,	///< 1 bit, 1=bit slice 1 is endpoint of product (minterm), 0=isn't.	
	_PIM_PMCFG_ENDP2	=2,	///< 1 bit, 1=bit slice 2 is endpoint of product (minterm), 0=isn't.	
	_PIM_PMCFG_ENDP3	=3,	///< 1 bit, 1=bit slice 3 is endpoint of product (minterm), 0=isn't.	
	_PIM_PMCFG_ENDP4	=4,	///< 1 bit, 1=bit slice 4 is endpoint of product (minterm), 0=isn't.	
	_PIM_PMCFG_ENDP5	=5,	///< 1 bit, 1=bit slice 5 is endpoint of product (minterm), 0=isn't.	
	_PIM_PMCFG_ENDP6	=6,	///< 1 bit, 1=bit slice 6 is endpoint of product (minterm), 0=isn't.	
	//_PIM_PMCFG_ENDP7	=7,	///< bit slice 7 is always endpoint of product (minterm).	

	_PIM_PMCFG_0		=8,	///< 3 bits, input source (0..7) for bit slice 0
	_PIM_PMCFG_1		=11,	///< 3 bits, input source for bit slice 1
	_PIM_PMCFG_2		=14,	///< 3 bits, input source for bit slice 2
	_PIM_PMCFG_3		=17,	///< 3 bits, input source for bit slice 3
	_PIM_PMCFG_4		=20,	///< 3 bits, input source for bit slice 4
	_PIM_PMCFG_5		=23,	///< 3 bits, input source for bit slice 5
	_PIM_PMCFG_6		=26,	///< 3 bits, input source for bit slice 6
	_PIM_PMCFG_7		=29,	///< 3 bits, input source for bit slice 7

	PIM_PMCFG_CONST1_		=0,
	PIM_PMCFG_STICKY_RISING_	=1,
	PIM_PMCFG_STICKY_FALLING_	=2,
	PIM_PMCFG_STICKY_BOTH_		=3,
	PIM_PMCFG_LEVEL1_		=4,
	PIM_PMCFG_LEVEL0_		=5,
	PIM_PMCFG_CONST0_		=6,
	PIM_PMCFG_EVENT_		=7,	///< one pulse version of BOTH
};

extern struct Pim PIM;		///< defined in crt0

#endif

