/*
  lpc8/usart.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

#ifndef lpc8__usart_h
#define lpc8__usart_h

#include <integers.h>
#include <stdbool.h>
#include <unionized.h>
#include <lpc8/syscon.h>

/** All registers of LPC800 USART are at most 16bits wide - so we use 16bit words in its register file.
 */
union PACKED4 Usart {
	//.0x00
	unionized struct PACKED { char _0[0x00]; // padding
	volatile Uint16		cfg;
	unionized };

	//.0x04
	unionized struct PACKED { char _1[0x04]; // padding
	volatile Uint16		ctrl;
	unionized };

	//.0x08
	unionized struct PACKED { char _2[0x08]; // padding
	volatile Uint16		stat;
	unionized };

	//.0x0C
	unionized struct PACKED { char _3[0x0C]; // padding
	volatile Uint16		intEnSet;
	unionized };

	//.0x10
	unionized struct PACKED { char _4[0x10]; // padding
	volatile Uint16		intEnClr;
	unionized };

	//.0x14
	unionized struct PACKED { char _5[0x14]; // padding
	const volatile Uint16	rxData;
	unionized };

	//.0x18
	unionized struct PACKED { char _6[0x18]; // padding
	const volatile Uint16	rxDataStat;
	unionized };

	//.0x1C
	unionized struct PACKED { char _7[0x1C]; // padding
	volatile Uint16		txData;
	unionized };

	//.0x20
	unionized struct PACKED { char _8[0x20]; // padding
	volatile Uint16		brg;		///< this value+1 is the divider for FRG clock
	unionized };

	//.0x24
	unionized struct PACKED { char _9[0x24]; // padding
	const volatile Uint16	intStat;
	unionized };
};

extern union Usart USART0;	///< defined in crt0
extern union Usart USART1;	///< defined in crt0

enum UsartCfgBits {
	_USART_CFG_ENABLE	=0,
	_USART_CFG_DATALEN	=2,	///< 2 bits: 0=7bits, 1=8bits, 2=9bits, 3=reserved
	_USART_CFG_PARITYSEL	=4,	///< 2 bits: 0=no, 1=reserved, 2=even, 3=odd
	_USART_CFG_STOPLEN	=6,	///< 1 bit: 0=1 stop bit, 1=2 stop bits
	_USART_CFG_CTSEN	=9,	///< 1 bit: 0=no flow control, 1=CTS flow control for transmitter
	_USART_CFG_SYNCEN	=11,	///< 1 bit: 0=async. , 1=sync mode
	_USART_CFG_CLKPOL	=12,	///< 1 bit: 0=RXD sampled on falling edge, 1=RXD sampled on rising edge
	_USART_CFG_SYNCMST	=14,	///< 1 bit: 0=slave in sync mode, 1=master in sync mode
	_USART_CFG_LOOP		=15,	///< 1 bit: 0=normal operation, 1=loopback mode
};

enum UsartCtrlBits {
	_USART_CTRL_TXBRKEN	=1,	///< 1 bit: 0=normal operation, 1=continuous break until cleared
	_USART_CTRL_ADDRDET	=2,	///< 0=receiver enabled for all incoming data, 1=only data with MSB=1
	_USART_CTRL_TXDIS	=6,	///< 0=normal operation, 1=transmit disable
	_USART_CTRL_CC		=8,	///< 0=clock on character only, 1=continuous clock
	_USART_CTRL_CLRCC	=9,	///< clear CC after receiving complete char.
};

enum UsartStatBits {
	_USART_STAT_RXRDY	=0,	///< 1=data can be read from RXDATA
	_USART_STAT_RXIDLE	=1,	///< 0=receiver is receiving, 1=receiver idle
	_USART_STAT_TXRDY	=2,	///< 1=data may be written into TXDATA
	_USART_STAT_TXIDLE	=3,	///< 0=transmission in progress, 1=transmitter idle
	_USART_STAT_CTS		=4,	///< level of CTS signal
	_USART_STAT_DELTACTS	=5,	///< 1=CTS has changed; access W1
	_USART_STAT_TXDISINT	=6,	///< transmitter disable interrupt flag
	_USART_STAT_OVERRUNINT	=8,	///< overrun error interrupt flag
	_USART_STAT_RXBRK	=10,	///< receive break (RXD low for 16 bits)
	_USART_STAT_DELTARXBRK	=11,
	_USART_STAT_START	=12,	///< start detected (for wakeup)
	_USART_STAT_FRAMERRINT	=13,	///< framing error interrupt flag
	_USART_STAT_PARITYERRINT=14,	///< parity error; access W1
	_USART_STAT_RXNOISEINT	=15,	///< noise detected
};

/** for intEnSet/inEnClr/intStat and bits 13..15 for RXDATASTAT, too */
enum UsartIntBits {
	_USART_INT_EN_RXRDY	=0,
	_USART_INT_EN_TXRDY	=2,
	_USART_INT_EN_DELTACTS	=5,
	_USART_INT_EN_TXDISINT	=6,
	_USART_INT_EN_OVERRUN	=8,
	_USART_INT_EN_DELTARXBRK=11,
	_USART_INT_EN_START	=12,
	_USART_INT_EN_FRAMERR	=13,
	_USART_INT_EN_PARITYERR	=14,
	_USART_INT_EN_RXNOISE	=15,
};

static inline bool usart0CanRead(void) {
	return 0!= (USART0.stat & 1<<_USART_STAT_RXRDY);
}

static inline bool usart0CanWrite(void) {
	return 0!= (USART0.stat & 1<<_USART_STAT_TXRDY);
}

/** Returns the next character. Check if a character has been received before calling this function.
 * @return a character as an unsigned 7..9bit value.
 */
static inline int usart0Read(void) {
	return USART0.rxData;
}

/** Writes the next character. Check if it can be sent before calling this functions.
 * @param c a character as a signed or unsigned 7..9bit value.
 */
static inline void usart0Write(int c) {
	USART0.txData = (Uint16)c;
}

int usart0BlockingRead(void);

void usart0BlockingWrite(int c);
void usart0BlockingWriteChar(char c);		///< compatible to print()

enum {
	BAUD_230400_FROM_12MHZ	=3,
	BAUD_230400_FROM_3MHZ	=1,

	BAUD_115200_FROM_12MHZ	=6,
	BAUD_115200_FROM_6MHZ	=3,
	BAUD_115200_FROM_4MHZ	=2,
	BAUD_115200_FROM_2MHZ	=1,

	BAUD_57600_FROM_12MHZ	=12,
	BAUD_57600_FROM_6MHZ	=6,
	BAUD_57600_FROM_4MHZ	=4,
	BAUD_57600_FROM_3MHZ	=3,
	BAUD_57600_FROM_2MHZ	=2,
	BAUD_57600_FROM_1MHZ	=1,

	BAUD_38400_FROM_12MHZ	=18,
	BAUD_38400_FROM_6MHZ	=9,
	BAUD_38400_FROM_4MHZ	=6,
	BAUD_38400_FROM_2MHZ	=3,

	BAUD_19200_FROM_12MHZ	=36,
	BAUD_19200_FROM_6MHZ	=18,
	BAUD_19200_FROM_4MHZ	=12,
	BAUD_19200_FROM_3MHZ	=9,
	BAUD_19200_FROM_2MHZ	=6,
	BAUD_19200_FROM_1MHZ	=3,
};

/** Initializes UART for 6*115200/divider baud. This function modifies the global fractional USART clock divider of the
 * LPC800. AHB clock must be enabled before for SWM and USART and USART pins should must be connected by the user.
 * @param divider the divider to derive the final baud rate. Most probably you want to use BAUD_115200_FROM_12MHZ
 * here.
 */
inline static void usart0InitFrom12Mhz(int divider) {
	// 12MHz = 115200 bd (*16) * 6 * (1+22/256) + small error
	SYSCON.usartClkDiv = divider;	// 0 = disable clock, 1=115200, 2=57600, 3=38400, ...
	SYSCON.usartFrgDiv = 255;	// yields the only supported divider of 256
	SYSCON.usartFrgMul = 22;

	//SWM.pinAssign0	= 0xFFFF0004;		// RXD and TXD on (ISP) default pins
	//USART0.brg	= 6-1;			// divider for 115200
	USART0.brg	= 1-1;			// divider for 6*115200 from 12MHz
	USART0.cfg	= 1<<_USART_CFG_ENABLE
			| 1<<_USART_CFG_DATALEN
			| 0<<_USART_CFG_PARITYSEL
			| 0<<_USART_CFG_STOPLEN
			;
}

#endif

